/*
 * This file is part of XFellow.
 *
 * kbddrv.c - Keyboard driver
 * Copyright 2000-2001 by Dan Sutherland <dan@dillinja.org>
 *
 * A lot of this file, mainly the PC->Amiga keymapping is based on the Windows
 * version by Marco Nova.
 *
 * This file also handles mouse and joystick events. That really needs to be
 * moved into a seperate file (events.c?).
 *
 * TODO:
 * - joykeys + autofire
 * - test that all keys actually work
 */

#include <SDL/SDL.h>
#include <SDL/SDL_thread.h>

#include "portable.h"
#include "defs.h"
#include "fellow.h"
#include "kbd.h"
#include "keycodes.h"
#include "gameport.h"
#include "gfxdrvmutex.h"
#include "kbddrv.h"

SDL_Thread *input_thread;

typedef enum {
  JOYKEY_LEFT = 0,
  JOYKEY_RIGHT = 1,
  JOYKEY_UP = 2,
  JOYKEY_DOWN = 3,
  JOYKEY_FIRE0 = 4,
  JOYKEY_FIRE1 = 5,
  JOYKEY_AUTOFIRE0 = 6,
  JOYKEY_AUTOFIRE1 = 7
} kbd_drv_joykey_directions;

#define MAX_JOYKEY_VALUE	8
//#define MAX_KEYS		256
#define MAX_KEYS		321

BOOLE			kbd_drv_joykey_enabled[2][2];
kbd_event		kbd_drv_joykey_event[2][2][MAX_JOYKEY_VALUE];

kbd_drv_pc_symbol        kbd_drv_joykey[2][MAX_JOYKEY_VALUE];
kbd_drv_pc_symbol        kbddrv_SDLK_to_symbol[MAX_KEYS];

UBY kbd_drv_pc_symbol_to_amiga_scancode[106] = {
  /* 0x00 */

  A_NONE,           /* PCK_NONE            */
  A_ESCAPE,         /* PCK_ESCAPE          */
  A_F1,             /* PCK_F1              */
  A_F2,             /* PCK_F2              */

  A_F3,             /* PCK_F3              */
  A_F4,             /* PCK_F4              */
  A_F5,             /* PCK_F5              */
  A_F6,             /* PCK_F6              */

  A_F7,             /* PCK_F7              */
  A_F8,             /* PCK_F8              */
  A_F9,             /* PCK_F9              */
  A_F10,            /* PCK_F10             */

  A_NONE,           /* PCK_F11             */
  A_NONE,           /* PCK_F12             */
  A_NONE,           /* PCK_PRINT_SCREEN    */
  A_BACKSLASH2,     /* PCK_SCROLL_LOCK     */

  /* 0x10 */

  A_NUMPAD_RIGHT_BRACKET,/* PCK_PAUSE           */
  A_GRAVE,          /* PCK_GRAVE           */
  A_1,              /* PCK_1               */
  A_2,              /* PCK_2               */

  A_3,              /* PCK_3               */
  A_4,              /* PCK_4               */
  A_5,              /* PCK_5               */
  A_6,              /* PCK_6               */

  A_7,              /* PCK_7               */
  A_8,              /* PCK_8               */
  A_9,              /* PCK_9               */
  A_0,              /* PCK_0               */

  A_MINUS,          /* PCK_MINUS           */
  A_EQUALS,         /* PCK_EQUALS          */
  A_BACKSPACE,      /* PCK_BACKSPACE       */
  A_HELP,           /* PCK_INSERT          */

  /* 0x20 */

  A_NONE,           /* PCK_HOME            */
  A_RIGHT_AMIGA,    /* PCK_PAGE_UP         */
  A_NUMPAD_LEFT_BRACKET,/* PCK_NUM_LOCK        */
  A_NUMPAD_DIVIDE,  /* PCK_NUMPAD_DIVIDE   */

  A_NUMPAD_MULTIPLY,/* PCK_NUMPAD_MULTIPLY */
  A_NUMPAD_MINUS,   /* PCK_NUMPAD_MINUS    */
  A_TAB,            /* PCK_TAB             */
  A_Q,              /* PCK_Q               */

  A_W,              /* PCK_W               */
  A_E,              /* PCK_E               */
  A_R,              /* PCK_R               */
  A_T,              /* PCK_T               */

  A_Y,              /* PCK_Y               */
  A_U,              /* PCK_U               */
  A_I,              /* PCK_I               */
  A_O,              /* PCK_O               */

  /* 0x30 */

  A_P,              /* PCK_P               */
  A_LEFT_BRACKET,   /* PCK_LBRACKET        */
  A_RIGHT_BRACKET,  /* PCK_RBRACKET        */
  A_RETURN,         /* PCK_RETURN          */

  A_DELETE,         /* PCK_DELETE          */
  A_NONE,           /* PCK_END             */
  A_LEFT_AMIGA,     /* PCK_PAGE_DOWN       */
  A_NUMPAD_7,       /* PCK_NUMPAD_7        */

  A_NUMPAD_8,       /* PCK_NUMPAD_8        */
  A_NUMPAD_9,       /* PCK_NUMPAD_9        */
  A_NUMPAD_PLUS,    /* PCK_NUMPAD_PLUS     */
  A_CAPS_LOCK,      /* PCK_CAPS_LOCK       */

  A_A,              /* PCK_A               */
  A_S,              /* PCK_S               */
  A_D,              /* PCK_D               */
  A_F,              /* PCK_F               */

  /* 0x40 */

  A_G,              /* PCK_G               */
  A_H,              /* PCK_H               */
  A_J,              /* PCK_J               */
  A_K,              /* PCK_K               */

  A_L,              /* PCK_L               */
  A_SEMICOLON,      /* PCK_SEMICOLON       */
  A_APOSTROPHE,     /* PCK_APOSTROPHE      */
  A_BACKSLASH,      /* PCK_BACKSLASH       */

  A_NUMPAD_4,       /* PCK_NUMPAD_4        */
  A_NUMPAD_5,       /* PCK_NUMPAD_5        */
  A_NUMPAD_6,       /* PCK_NUMPAD_6        */
  A_LEFT_SHIFT,     /* PCK_LEFT_SHIFT      */

  A_LESS_THAN,      /* PCK_NONAME1         */
  A_Z,              /* PCK_Z               */
  A_X,              /* PCK_X               */
  A_C,              /* PCK_C               */

  /* 0x50 */

  A_V,              /* PCK_V               */
  A_B,              /* PCK_B               */
  A_N,              /* PCK_N               */
  A_M,              /* PCK_M               */

  A_COMMA,          /* PCK_COMMA           */
  A_PERIOD,         /* PCK_PERIOD          */
  A_SLASH,          /* PCK_SLASH           */
  A_RIGHT_SHIFT,    /* PCK_RIGHT_SHIFT     */

  A_UP,             /* PCK_UP              */
  A_NUMPAD_1,       /* PCK_NUMPAD_1        */
  A_NUMPAD_2,       /* PCK_NUMPAD_2        */
  A_NUMPAD_3,       /* PCK_NUMPAD_3        */

  A_NUMPAD_ENTER,   /* PCK_NUMPAD_ENTER    */
  A_LEFT_AMIGA,     /* PCK_LEFT_CTRL       */
  A_LEFT_AMIGA,     /* PCK_LEFT_WINDOWS    */
  A_LEFT_ALT,       /* PCK_LEFT_ALT        */

  /* 0x60 */

  A_SPACE,          /* PCK_SPACE           */
  A_RIGHT_ALT,      /* PCK_RIGHT_ALT       */
  A_RIGHT_AMIGA,    /* PCK_RIGHT_WINDOWS   */
  A_NONE,           /* PCK_START_MENU      */

  A_CTRL,           /* PCK_RIGHT_CTRL      */
  A_LEFT,           /* PCK_LEFT            */
  A_DOWN,           /* PCK_DOWN            */
  A_RIGHT,          /* PCK_RIGHT           */

  A_NUMPAD_0,       /* PCK_NUMPAD_0        */
  A_NUMPAD_DOT      /* PCK_NUMPAD_DOT      */
};

STR *kbd_drv_pc_symbol_to_string[106] = {
  "NONE",
  "ESCAPE",
  "F1",
  "F2",
  "F3",
  "F4",
  "F5",
  "F6",
  "F7",
  "F8",
  "F9",
  "F10",
  "F11",
  "F12",
  "PRINT_SCREEN",
  "SCROLL_LOCK",
  "PAUSE",
  "GRAVE",
  "1",
  "2",
  "3",
  "4",
  "5",
  "6",
  "7",
  "8",
  "9",
  "0",
  "MINUS",
  "EQUALS",
  "BACKSPACE",
  "INSERT",
  "HOME",
  "PAGE_UP",
  "NUM_LOCK",
  "NUMPAD_DIVIDE",
  "NUMPAD_MULTIPLY",
  "NUMPAD_MINUS",
  "TAB",
  "Q",
  "W",
  "E",
  "R",
  "T",
  "Y",
  "U",
  "I",
  "O",
  "P",
  "LEFT_BRACKET",
  "RIGHT_BRACKET",
  "RETURN",
  "DELETE",
  "END",
  "PAGE_DOWN",
  "NUMPAD_7",
  "NUMPAD_8",
  "NUMPAD_9",
  "NUMPAD_PLUS",
  "CAPS_LOCK",
  "A",
  "S",
  "D",
  "F",
  "G",
  "H",
  "J",
  "K",
  "L",
  "SEMICOLON",
  "APOSTROPHE",
  "BACKSLASH",
  "NUMPAD_4",
  "NUMPAD_5",
  "NUMPAD_6",
  "LEFT_SHIFT",
  "NONAME1",
  "Z",
  "X",
  "C",
  "V",
  "B",
  "N",
  "M",
  "COMMA",
  "PERIOD",
  "SLASH",
  "RIGHT_SHIFT",
  "UP",
  "NUMPAD_1",
  "NUMPAD_2",
  "NUMPAD_3",
  "NUMPAD_ENTER",
  "LEFT_CTRL",
  "LEFT_WINDOWS",
  "LEFT_ALT",
  "SPACE",
  "RIGHT_ALT",
  "RIGHT_WINDOWS",
  "RIGHT_MENU",
  "RIGHT_CTRL",
  "LEFT",
  "DOWN",
  "RIGHT",
  "NUMPAD_0",
  "NUMPAD_DOT"};



int symbol_to_SDLK_kbddrv[PCK_LAST_KEY] = {
  0,
  SDLK_ESCAPE,
  SDLK_F1,
  SDLK_F2,
  SDLK_F3,
  SDLK_F4,
  SDLK_F5,
  SDLK_F6,
  SDLK_F7,
  SDLK_F8,
  SDLK_F9,
  SDLK_F10,
  SDLK_F11,
  SDLK_F12,
  SDLK_SYSREQ,
  SDLK_SCROLLOCK,
  0 /*SDLK_PAUSE*/,
  SDLK_BACKQUOTE,
  SDLK_1,
  SDLK_2,
  SDLK_3,
  SDLK_4,
  SDLK_5,
  SDLK_6,
  SDLK_7,
  SDLK_8,
  SDLK_9,
  SDLK_0,
  SDLK_MINUS,
  SDLK_EQUALS,
  SDLK_BACKSPACE,
  SDLK_INSERT,
  SDLK_HOME,
  SDLK_PAGEUP,
  SDLK_NUMLOCK,
  SDLK_KP_DIVIDE,
  SDLK_KP_MULTIPLY,
  SDLK_KP_MINUS,
  SDLK_TAB,
  SDLK_q,
  SDLK_w,
  SDLK_e,
  SDLK_r,
  SDLK_t,
  SDLK_y,
  SDLK_u,
  SDLK_i,
  SDLK_o,
  SDLK_p,
  SDLK_LEFTBRACKET,
  SDLK_RIGHTBRACKET,
  SDLK_RETURN,
  SDLK_DELETE,
  SDLK_END,
  SDLK_PAGEDOWN,
  SDLK_KP7,
  SDLK_KP8,
  SDLK_KP9,
  SDLK_KP_PLUS,
  SDLK_CAPSLOCK,
  SDLK_a,
  SDLK_s,
  SDLK_d,
  SDLK_f,
  SDLK_g,
  SDLK_h,
  SDLK_j,
  SDLK_k,
  SDLK_l,
  SDLK_SEMICOLON,
  SDLK_QUOTE,
  SDLK_BACKSLASH,
  SDLK_KP4,
  SDLK_KP5,
  SDLK_KP6,
  SDLK_LSHIFT,
  0x56,  /* Not defined */
  SDLK_z,
  SDLK_x,
  SDLK_c,
  SDLK_v,
  SDLK_b,
  SDLK_n,
  SDLK_m,
  SDLK_COMMA,
  SDLK_PERIOD,
  SDLK_SLASH,
  SDLK_RSHIFT,
  SDLK_UP,
  SDLK_KP1,
  SDLK_KP2,
  SDLK_KP3,
  SDLK_KP_ENTER,
  SDLK_LCTRL,
  SDLK_LSUPER,
  SDLK_LMETA,
  SDLK_SPACE,
  SDLK_RMETA,
  SDLK_RSUPER,
  SDLK_MENU,
  SDLK_RCTRL,
  SDLK_LEFT,
  SDLK_DOWN,
  SDLK_RIGHT,
  SDLK_KP0,
  SDLK_KP_PERIOD
};

/****************************************************************************/
void kbdDrvInputThread(void *data)
{
	static SDL_Event *event;
	static int skey;
	static BOOLE btnState[2];

	fellowAddLog("kbdDrvInputThread(): input thread starting\n");

	btnState[0] = btnState[1] = FALSE;

	event = malloc(sizeof(SDL_Event));

	while(SDL_WaitEvent(event)) {
		SDL_mutexP(gfxDrvMutex);
		switch (event->type) {
		case SDL_MOUSEMOTION:
//			fprintf(stderr, "mousemotion event\n");
			gameportMouseHandler(GP_MOUSE0,
				((SDL_MouseMotionEvent *)event)->xrel,
				((SDL_MouseMotionEvent *)event)->yrel, btnState[0], FALSE, btnState[1]);
			break;
		case SDL_KEYDOWN:
			skey = kbddrv_SDLK_to_symbol[((SDL_KeyboardEvent *)event)->keysym.sym];
//			fprintf(stderr, "keydown event %i:%i\n", ((SDL_KeyboardEvent *)event)->keysym.sym, skey);

			kbdKeyAdd(kbd_drv_pc_symbol_to_amiga_scancode[skey]);
			break;
		case SDL_KEYUP:
//			fprintf(stderr, "keyup event\n");
			skey = kbddrv_SDLK_to_symbol[((SDL_KeyboardEvent *)event)->keysym.sym];

			if(skey == PCK_F12) {
				fellowAddLog("kbdDrvInputThread(): input thread exiting\n");
				fellowRequestEmulationStop();
				return;
			}
			kbdKeyAdd(kbd_drv_pc_symbol_to_amiga_scancode[skey] | 0x80);
			break;
		case SDL_MOUSEBUTTONDOWN:
//			fprintf(stderr, "mousebuttondown event\n");
			if (((SDL_MouseButtonEvent *)event)->button == SDL_BUTTON_LEFT) {
				btnState[0] = TRUE;
			} else {
				btnState[1] = TRUE;
			}
			gameportMouseHandler(GP_MOUSE0, 0, 0, btnState[0], FALSE, btnState[1]);
			break;
		case SDL_MOUSEBUTTONUP:
//			fprintf(stderr, "mousebuttonup event\n");
			if (((SDL_MouseButtonEvent *)event)->button == SDL_BUTTON_LEFT) {
				btnState[0] = FALSE;
			} else {
				btnState[1] = FALSE;
			}
			gameportMouseHandler(GP_MOUSE0, 0, 0, btnState[0], FALSE, btnState[1]);
			break;
		case SDL_JOYBUTTONDOWN:
		case SDL_JOYBUTTONUP:
//			fprintf(stderr, "joy %i, %i\n", ((SDL_JoyButtonEvent *)event)->which, ((SDL_JoyButtonEvent *)event)->state);
			break;
		default:
			fprintf(stderr, "unhandled event\n");
		}
		SDL_mutexV(gfxDrvMutex);
	}

	free(event);
}

STR *kbdDrvKeyString(ULO symbolickey)
{

}

void kbdDrvJoyReplacementSet(kbd_event event, ULO symbolickey)
{

}

ULO kbdDrvJoyReplacementGet(kbd_event event)
{

}

void kbdDrvHardReset(void)
{

}

void kbdDrvEmulationStart(void)
{
	fellowAddLog("kbdDrvEmulationStart(): starting input thread\n");
	input_thread = SDL_CreateThread(kbdDrvInputThread, NULL);
}

void kbdDrvEmulationStop(void)
{
	/* if the input thread is running (it shouldn't be), then terminate it */
	// FIXME: do this
}

void kbdDrvStartup(void)
{
	ULO port, setting;
	kbd_drv_joykey_directions direction;
	int i;

	fellowAddLog("kbdDrvStartup(): initialising\n");

	kbd_drv_joykey_event[0][0][JOYKEY_UP] = EVENT_JOY0_UP_INACTIVE;
  kbd_drv_joykey_event[0][1][JOYKEY_UP] = EVENT_JOY0_UP_ACTIVE;
  kbd_drv_joykey_event[1][0][JOYKEY_UP] = EVENT_JOY1_UP_INACTIVE;
  kbd_drv_joykey_event[1][1][JOYKEY_UP] = EVENT_JOY1_UP_ACTIVE;
  kbd_drv_joykey_event[0][0][JOYKEY_DOWN] = EVENT_JOY0_DOWN_INACTIVE;
  kbd_drv_joykey_event[0][1][JOYKEY_DOWN] = EVENT_JOY0_DOWN_ACTIVE;
  kbd_drv_joykey_event[1][0][JOYKEY_DOWN] = EVENT_JOY1_DOWN_INACTIVE;
  kbd_drv_joykey_event[1][1][JOYKEY_DOWN] = EVENT_JOY1_DOWN_ACTIVE;
  kbd_drv_joykey_event[0][0][JOYKEY_LEFT] = EVENT_JOY0_LEFT_INACTIVE;
  kbd_drv_joykey_event[0][1][JOYKEY_LEFT] = EVENT_JOY0_LEFT_ACTIVE;
  kbd_drv_joykey_event[1][0][JOYKEY_LEFT] = EVENT_JOY1_LEFT_INACTIVE;
  kbd_drv_joykey_event[1][1][JOYKEY_LEFT] = EVENT_JOY1_LEFT_ACTIVE;
  kbd_drv_joykey_event[0][0][JOYKEY_RIGHT] = EVENT_JOY0_RIGHT_INACTIVE;
  kbd_drv_joykey_event[0][1][JOYKEY_RIGHT] = EVENT_JOY0_RIGHT_ACTIVE;
  kbd_drv_joykey_event[1][0][JOYKEY_RIGHT] = EVENT_JOY1_RIGHT_INACTIVE;
  kbd_drv_joykey_event[1][1][JOYKEY_RIGHT] = EVENT_JOY1_RIGHT_ACTIVE;
  kbd_drv_joykey_event[0][0][JOYKEY_FIRE0] = EVENT_JOY0_FIRE0_INACTIVE;
  kbd_drv_joykey_event[0][1][JOYKEY_FIRE0] = EVENT_JOY0_FIRE0_ACTIVE;
  kbd_drv_joykey_event[1][0][JOYKEY_FIRE0] = EVENT_JOY1_FIRE0_INACTIVE;
  kbd_drv_joykey_event[1][1][JOYKEY_FIRE0] = EVENT_JOY1_FIRE0_ACTIVE;
  kbd_drv_joykey_event[0][0][JOYKEY_FIRE1] = EVENT_JOY0_FIRE1_INACTIVE;
  kbd_drv_joykey_event[0][1][JOYKEY_FIRE1] = EVENT_JOY0_FIRE1_ACTIVE;
  kbd_drv_joykey_event[1][0][JOYKEY_FIRE1] = EVENT_JOY1_FIRE1_INACTIVE;
  kbd_drv_joykey_event[1][1][JOYKEY_FIRE1] = EVENT_JOY1_FIRE1_ACTIVE;
  kbd_drv_joykey_event[0][0][JOYKEY_AUTOFIRE0] = EVENT_JOY0_AUTOFIRE0_INACTIVE;
  kbd_drv_joykey_event[0][1][JOYKEY_AUTOFIRE0] = EVENT_JOY0_AUTOFIRE0_ACTIVE;
  kbd_drv_joykey_event[1][0][JOYKEY_AUTOFIRE0] = EVENT_JOY1_AUTOFIRE0_INACTIVE;
  kbd_drv_joykey_event[1][1][JOYKEY_AUTOFIRE0] = EVENT_JOY1_AUTOFIRE0_ACTIVE;
  kbd_drv_joykey_event[0][0][JOYKEY_AUTOFIRE1] = EVENT_JOY0_AUTOFIRE1_INACTIVE;
  kbd_drv_joykey_event[0][1][JOYKEY_AUTOFIRE1] = EVENT_JOY0_AUTOFIRE1_ACTIVE;
  kbd_drv_joykey_event[1][0][JOYKEY_AUTOFIRE1] = EVENT_JOY1_AUTOFIRE1_INACTIVE;
  kbd_drv_joykey_event[1][1][JOYKEY_AUTOFIRE1] = EVENT_JOY1_AUTOFIRE1_ACTIVE;

  kbd_drv_joykey[0][JOYKEY_UP]    = SDLK_UP;
  kbd_drv_joykey[0][JOYKEY_DOWN]  = SDLK_DOWN;
  kbd_drv_joykey[0][JOYKEY_LEFT]  = SDLK_LEFT;
  kbd_drv_joykey[0][JOYKEY_RIGHT] = SDLK_RIGHT;
  kbd_drv_joykey[0][JOYKEY_FIRE0] = SDLK_RCTRL;
  kbd_drv_joykey[0][JOYKEY_FIRE1] = SDLK_MENU;
  kbd_drv_joykey[0][JOYKEY_AUTOFIRE0] = SDLK_o;
  kbd_drv_joykey[0][JOYKEY_AUTOFIRE1] = SDLK_p;
  kbd_drv_joykey[1][JOYKEY_UP]    = SDLK_r;
  kbd_drv_joykey[1][JOYKEY_DOWN]  = SDLK_f;
  kbd_drv_joykey[1][JOYKEY_LEFT]  = SDLK_d;
  kbd_drv_joykey[1][JOYKEY_RIGHT] = SDLK_g;
  kbd_drv_joykey[1][JOYKEY_FIRE0] = SDLK_LCTRL;
  kbd_drv_joykey[1][JOYKEY_FIRE1] = SDLK_LSUPER;
  kbd_drv_joykey[1][JOYKEY_AUTOFIRE0] = SDLK_a;
  kbd_drv_joykey[1][JOYKEY_AUTOFIRE1] = SDLK_s;

	for (port = 0; port < 2; port++)
		for (setting = 0; setting < 2; setting++)
			kbd_drv_joykey_enabled[port][setting] = FALSE;

	for( i = 0; i < PCK_LAST_KEY; i++ )
		kbddrv_SDLK_to_symbol[ i ] = PCK_NONE;

  kbddrv_SDLK_to_symbol[ SDLK_ESCAPE		] = PCK_ESCAPE; /* First row */
  kbddrv_SDLK_to_symbol[ SDLK_F1		] = PCK_F1;
  kbddrv_SDLK_to_symbol[ SDLK_F2		] = PCK_F2;
  kbddrv_SDLK_to_symbol[ SDLK_F3		] = PCK_F3;
  kbddrv_SDLK_to_symbol[ SDLK_F4		] = PCK_F4;
  kbddrv_SDLK_to_symbol[ SDLK_F5		] = PCK_F5;
  kbddrv_SDLK_to_symbol[ SDLK_F6		] = PCK_F6;
  kbddrv_SDLK_to_symbol[ SDLK_F7		] = PCK_F7;
  kbddrv_SDLK_to_symbol[ SDLK_F8		] = PCK_F8;
  kbddrv_SDLK_to_symbol[ SDLK_F9		] = PCK_F9;
  kbddrv_SDLK_to_symbol[ SDLK_F10		] = PCK_F10;
  kbddrv_SDLK_to_symbol[ SDLK_F11		] = PCK_F11;
  kbddrv_SDLK_to_symbol[ SDLK_F12		] = PCK_F12;
  kbddrv_SDLK_to_symbol[ SDLK_PRINT		] = PCK_PRINT_SCREEN;
  kbddrv_SDLK_to_symbol[ SDLK_SCROLLOCK		] = PCK_SCROLL_LOCK;

  /*kbddrv_SDLK_to_symbol[ SDLK_PAUSE           ] = PCK_PAUSE; */

  kbddrv_SDLK_to_symbol[ SDLK_BACKQUOTE		] = PCK_GRAVE;  /* Second row */
  kbddrv_SDLK_to_symbol[ SDLK_1			] = PCK_1;
  kbddrv_SDLK_to_symbol[ SDLK_2			] = PCK_2;
  kbddrv_SDLK_to_symbol[ SDLK_3			] = PCK_3;
  kbddrv_SDLK_to_symbol[ SDLK_4			] = PCK_4;
  kbddrv_SDLK_to_symbol[ SDLK_5			] = PCK_5;
  kbddrv_SDLK_to_symbol[ SDLK_6			] = PCK_6;
  kbddrv_SDLK_to_symbol[ SDLK_7			] = PCK_7;
  kbddrv_SDLK_to_symbol[ SDLK_8			] = PCK_8;
  kbddrv_SDLK_to_symbol[ SDLK_9			] = PCK_9;
  kbddrv_SDLK_to_symbol[ SDLK_0			] = PCK_0;
  kbddrv_SDLK_to_symbol[ SDLK_MINUS		] = PCK_MINUS;
  kbddrv_SDLK_to_symbol[ SDLK_EQUALS		] = PCK_EQUALS;
  kbddrv_SDLK_to_symbol[ SDLK_BACKSPACE		] = PCK_BACKSPACE;
  kbddrv_SDLK_to_symbol[ SDLK_INSERT		] = PCK_INSERT;
  kbddrv_SDLK_to_symbol[ SDLK_HOME		] = PCK_HOME;
  kbddrv_SDLK_to_symbol[ SDLK_PAGEUP		] = PCK_PAGE_UP;
  kbddrv_SDLK_to_symbol[ SDLK_NUMLOCK		] = PCK_NUM_LOCK;
  kbddrv_SDLK_to_symbol[ SDLK_KP_DIVIDE		] = PCK_NUMPAD_DIVIDE;
  kbddrv_SDLK_to_symbol[ SDLK_KP_MULTIPLY	] = PCK_NUMPAD_MULTIPLY;
  kbddrv_SDLK_to_symbol[ SDLK_KP_MINUS		] = PCK_NUMPAD_MINUS;
  kbddrv_SDLK_to_symbol[ SDLK_TAB		] = PCK_TAB;  /* Third row */
  kbddrv_SDLK_to_symbol[ SDLK_q			] = PCK_Q;
  kbddrv_SDLK_to_symbol[ SDLK_w			] = PCK_W;
  kbddrv_SDLK_to_symbol[ SDLK_e			] = PCK_E;
  kbddrv_SDLK_to_symbol[ SDLK_r			] = PCK_R;
  kbddrv_SDLK_to_symbol[ SDLK_t			] = PCK_T;
  kbddrv_SDLK_to_symbol[ SDLK_y			] = PCK_Y;
  kbddrv_SDLK_to_symbol[ SDLK_u			] = PCK_U;
  kbddrv_SDLK_to_symbol[ SDLK_i			] = PCK_I;
  kbddrv_SDLK_to_symbol[ SDLK_o			] = PCK_O;
  kbddrv_SDLK_to_symbol[ SDLK_p			] = PCK_P;
  kbddrv_SDLK_to_symbol[ SDLK_LEFTBRACKET	] = PCK_LBRACKET;
  kbddrv_SDLK_to_symbol[ SDLK_RIGHTBRACKET	] = PCK_RBRACKET;
  kbddrv_SDLK_to_symbol[ SDLK_RETURN		] = PCK_RETURN;
  kbddrv_SDLK_to_symbol[ SDLK_DELETE		] = PCK_DELETE;
  kbddrv_SDLK_to_symbol[ SDLK_END		] = PCK_END;
  kbddrv_SDLK_to_symbol[ SDLK_PAGEDOWN		] = PCK_PAGE_DOWN;
  kbddrv_SDLK_to_symbol[ SDLK_KP7		] = PCK_NUMPAD_7;
  kbddrv_SDLK_to_symbol[ SDLK_KP8		] = PCK_NUMPAD_8;
  kbddrv_SDLK_to_symbol[ SDLK_KP9		] = PCK_NUMPAD_9;
  kbddrv_SDLK_to_symbol[ SDLK_KP_PLUS		] = PCK_NUMPAD_PLUS;

  kbddrv_SDLK_to_symbol[ SDLK_CAPSLOCK		] = PCK_CAPS_LOCK; /* Fourth row */
  kbddrv_SDLK_to_symbol[ SDLK_a			] = PCK_A;
  kbddrv_SDLK_to_symbol[ SDLK_s			] = PCK_S;
  kbddrv_SDLK_to_symbol[ SDLK_d			] = PCK_D;
  kbddrv_SDLK_to_symbol[ SDLK_f			] = PCK_F;
  kbddrv_SDLK_to_symbol[ SDLK_g			] = PCK_G;
  kbddrv_SDLK_to_symbol[ SDLK_h			] = PCK_H;
  kbddrv_SDLK_to_symbol[ SDLK_j			] = PCK_J;
  kbddrv_SDLK_to_symbol[ SDLK_k			] = PCK_K;
  kbddrv_SDLK_to_symbol[ SDLK_l			] = PCK_L;
  kbddrv_SDLK_to_symbol[ SDLK_SEMICOLON		] = PCK_SEMICOLON;
  kbddrv_SDLK_to_symbol[ SDLK_QUOTE		] = PCK_APOSTROPHE;
  kbddrv_SDLK_to_symbol[ SDLK_BACKSLASH		] = PCK_BACKSLASH;
  kbddrv_SDLK_to_symbol[ SDLK_KP4		] = PCK_NUMPAD_4;
  kbddrv_SDLK_to_symbol[ SDLK_KP5		] = PCK_NUMPAD_5;
  kbddrv_SDLK_to_symbol[ SDLK_KP6		] = PCK_NUMPAD_6;

  kbddrv_SDLK_to_symbol[ SDLK_LSHIFT		] = PCK_LEFT_SHIFT; /* Fifth row */
  kbddrv_SDLK_to_symbol[ 0x56			] = PCK_NONAME1;
  kbddrv_SDLK_to_symbol[ SDLK_z			] = PCK_Z;
  kbddrv_SDLK_to_symbol[ SDLK_x			] = PCK_X;
  kbddrv_SDLK_to_symbol[ SDLK_c			] = PCK_C;
  kbddrv_SDLK_to_symbol[ SDLK_v			] = PCK_V;
  kbddrv_SDLK_to_symbol[ SDLK_b			] = PCK_B;
  kbddrv_SDLK_to_symbol[ SDLK_n			] = PCK_N;
  kbddrv_SDLK_to_symbol[ SDLK_m			] = PCK_M;
  kbddrv_SDLK_to_symbol[ SDLK_COMMA		] = PCK_COMMA;
  kbddrv_SDLK_to_symbol[ SDLK_PERIOD		] = PCK_PERIOD;
  kbddrv_SDLK_to_symbol[ SDLK_SLASH		] = PCK_SLASH;
  kbddrv_SDLK_to_symbol[ SDLK_RSHIFT		] = PCK_RIGHT_SHIFT;
  kbddrv_SDLK_to_symbol[ SDLK_UP		] = PCK_UP;
  kbddrv_SDLK_to_symbol[ SDLK_KP1		] = PCK_NUMPAD_1;
  kbddrv_SDLK_to_symbol[ SDLK_KP2		] = PCK_NUMPAD_2;
  kbddrv_SDLK_to_symbol[ SDLK_KP3		] = PCK_NUMPAD_3;
  kbddrv_SDLK_to_symbol[ SDLK_KP_ENTER		] = PCK_NUMPAD_ENTER;

  kbddrv_SDLK_to_symbol[ SDLK_LCTRL		] = PCK_LEFT_CTRL; /* Sixth row */
  kbddrv_SDLK_to_symbol[ SDLK_LSUPER		] = PCK_LEFT_WINDOWS;
  kbddrv_SDLK_to_symbol[ SDLK_LMETA		] = PCK_LEFT_ALT;
  kbddrv_SDLK_to_symbol[ SDLK_SPACE		] = PCK_SPACE;
  kbddrv_SDLK_to_symbol[ SDLK_RMETA		] = PCK_RIGHT_ALT;
  kbddrv_SDLK_to_symbol[ SDLK_RSUPER		] = PCK_RIGHT_WINDOWS;
  kbddrv_SDLK_to_symbol[ SDLK_MENU		] = PCK_START_MENU;
  kbddrv_SDLK_to_symbol[ SDLK_RCTRL		] = PCK_RIGHT_CTRL;
  kbddrv_SDLK_to_symbol[ SDLK_LEFT		] = PCK_LEFT;
  kbddrv_SDLK_to_symbol[ SDLK_DOWN		] = PCK_DOWN;
  kbddrv_SDLK_to_symbol[ SDLK_RIGHT		] = PCK_RIGHT;
  kbddrv_SDLK_to_symbol[ SDLK_KP0		] = PCK_NUMPAD_0;
  kbddrv_SDLK_to_symbol[ SDLK_KP_PERIOD		] = PCK_NUMPAD_DOT;

  prsReadFile( "mapping.key", kbd_drv_pc_symbol_to_amiga_scancode, kbd_drv_joykey );

  for (port = 0; port < 2; port++)
    for (setting = 0; setting < MAX_JOYKEY_VALUE; setting++)
        {
          kbd_drv_joykey[port][setting] = kbddrv_SDLK_to_symbol[kbd_drv_joykey[port][setting]];
        }

}

void kbdDrvShutdown(void)
{
	fellowAddLog("kbdDrvShutdown(): was called\n");
}

