/*
 * sounddrv.c - Sound driver for XFellow (OSS).
 * Author: Dan Sutherland <ds@chromerhino.demon.co.uk>. GPL.
 *
 * TODO:
 * - support mono modes
 * - rewrite using SDL, and use a timer to sync the emulator even without sound
 */

#include "portable.h"
#include "defs.h"
#include "fellow.h"
#include "sound.h"

#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/soundcard.h>

#define SND_DEVICE "/dev/dsp"

/* some globals */
int soundDrvDSPfd;
WOR *soundDrvBuffer;

BOOLE soundDrvStartup(sound_device *devinfo)
{
	fellowAddLog("soundDrvStartup(): nop\n");
	return TRUE;
}

void soundDrvShutdown(void)
{
}

BOOLE soundDrvEmulationStart(ULO outputrate, BOOLE bits16, BOOLE stereo,
				ULO *buffersamplecountmax)
{
	int reqFmt, actFmt;
	int reqChan, actChan;
	int reqRate, actRate;
	int frag = 0x0002000c;

	fellowAddLog("soundDrvEmulationStart(): in\n");

	reqFmt = actFmt = bits16 ? AFMT_S16_LE : AFMT_U8;
	reqChan = actChan = stereo ? 2 : 1;
	reqRate = actRate = outputrate;

	/* open DSP device */
	if((soundDrvDSPfd = open(SND_DEVICE, O_WRONLY, 0)) == -1) {
		fellowAddLog("soundDrvEmulationStart(): couldn't open dsp\n");
		return FALSE;
	}

	/* set buffer size */
	if(ioctl(soundDrvDSPfd, SNDCTL_DSP_SETFRAGMENT, &frag)) {
		fellowAddLog("soundDrvEmulationStart(): SNDCTL_DSP_SETFRAGMENT failed\n");
	}

	/* set sample format */
	if(ioctl(soundDrvDSPfd, SNDCTL_DSP_SETFMT, &reqFmt) == -1) {
		fellowAddLog("soundDrvEmulationStart(): ioctl SNDCTL_DSP_SETFMT failed\n");
		return FALSE;
	}

	/* verify actual sample format is the one we specified */
	if(reqFmt != actFmt) {
		fellowAddLog("soundDrvEmulationStart(): actual sample format does not match selected\n");
		return FALSE;
	}

	/* set channels */
	if(ioctl(soundDrvDSPfd, SNDCTL_DSP_CHANNELS, &reqChan) == -1) {
		fellowAddLog("soundDrvEmulationStart(): ioctl SNDCTL_DSP_CHANNELS failed\n");
		return FALSE;
	}

	if(reqChan != actChan) {
		fellowAddLog("soundDrvEmulationStart(): didn't get specified channels\n");
		return FALSE;
	}

	/* set sample rate */
	if(ioctl(soundDrvDSPfd, SNDCTL_DSP_SPEED, &reqRate) == -1) {
		fellowAddLog("soundDrvEmulationStart(): ioctl SNDCTL_DSP_SPEED failed\n");
		return FALSE;
	}

	/* check we actally go it */
	if(reqRate != actRate) {
		fellowAddLog("soundDrvEmulationStart(): actual sample rate is not the one we specified\n");
		return FALSE;
	}

	/* allocate sound buffer */
	if((soundDrvBuffer = malloc((*buffersamplecountmax) * actChan * 2)) == NULL) {
		fellowAddLog("soundDrvEmulationStart(): couldn't allocate audio buffer\n");
		return FALSE;
	}

	fellowAddLog("soundDrvEmulationStart(): out\n");

	return TRUE;
}

void soundDrvEmulationStop(void)
{
	fellowAddLog("soundDrvEmulationStop(): closing sound device\n");
	close(soundDrvDSPfd);
}

void soundDrvPlay(WOR *leftbuffer, WOR *rightbuffer, ULO samplecount)
{
	ULO i;

	for (i = 0 ; i < samplecount * 2 ; i += 2) {
		*(soundDrvBuffer + (i * 2)) = *(leftbuffer + i);
		*(soundDrvBuffer + (i * 2) + 1) = *(leftbuffer + i + 1);
		*(soundDrvBuffer + (i * 2) + 2) = *(rightbuffer + i);
		*(soundDrvBuffer + (i * 2) + 3) = *(rightbuffer + i + 1);
	}

	write(soundDrvDSPfd, soundDrvBuffer, samplecount * 4);
}

